@@ -222,6 +222,72 @@ def trigfft(y, tr=-1):
     }
 
 
+def trigfft2(y):
+    """calculate trigonometric coefficients using FFT
+    Assuming the periodicity is 2*pi
+    params:
+        y -- 2D array for Fourier transformation
+    return:
+        a dict containing
+        'n' -- 1D array, n index
+        'm' -- 1D array, m index
+        'rcos' -- 2D array, cos coefficients of the real part
+        'rsin' -- 2D array, sin coefficients of the real part
+        'icos' -- 2D array, cos coefficients of the imag part
+        'isin' -- 2D array, sin coefficients of the imag part
+    """
+    from scipy.fftpack import fft2, fftshift
+
+    M, N = y.shape
+    mn = M * N
+    comp = fft2(y) / mn
+    if M % 2 == 0:  # even
+        half = M // 2 - 1
+        end = half + 2
+    else:
+        half = (M - 1) // 2
+        end = half + 1
+    if N % 2 == 0:  # even
+        mid0 = N // 2
+        start = 1
+        nmin = -N // 2
+        nmax = N // 2 - 1
+    else:
+        mid0 = (N - 1) // 2
+        start = 0
+        nmin = -(N - 1) // 2
+        nmax = (N - 1) // 2
+    a_k = np.zeros((end, N), dtype=np.complex)
+    b_k = np.zeros((end, N), dtype=np.complex)
+    # find mapping
+    a_k[0, 0] = comp[0, 0]
+    for n in range(1, N):
+        a_k[0, n] = comp[0, n] + comp[0, N - n]
+        b_k[0, n] = (comp[0, n] - comp[0, N - n]) * 1j
+    for m in range(1, (M + 1) // 2):
+        a_k[m, 0] = comp[m, 0] + comp[M - m, 0]
+        b_k[m, 0] = (comp[m, 0] - comp[M - m, 0]) * 1j
+        for n in range(1, N):
+            a_k[m, n] = comp[m, n] + comp[M - m, N - n]
+            b_k[m, n] = (comp[m, n] - comp[M - m, N - n]) * 1j
+    if M % 2 == 0 and N % 2 == 0:  # even
+        a_k[end - 1, N // 2] = comp[M // 2, N // 2]
+    a_k = fftshift(a_k, axes=1)
+    b_k = fftshift(b_k, axes=1)
+    a_k[0, start:mid0] = 0 + 1j * 0
+    b_k[0, start:mid0] = 0 + 1j * 0
+    mm = np.arange(end)
+    nn = np.arange(nmin, nmax + 1)
+    return {
+        "n": nn,
+        "m": mm,
+        "rcos": np.real(a_k),
+        "rsin": np.real(b_k),
+        "icos": np.imag(a_k),
+        "isin": np.imag(b_k),
+    }
+
+
 def fft_deriv(y):
     from scipy.fftpack import fft, ifft
 
@@ -238,6 +304,7 @@ def fft_deriv(y):
         ) * 1j
     return ifft(comp * dt)
 
+
 def trig2real(theta, zeta=None, xm=[], xn=[], fmnc=None, fmns=None):
     """Trigonometric coefficients to real space points
 
@@ -251,12 +318,13 @@ def trig2real(theta, zeta=None, xm=[], xn=[], fmnc=None, fmns=None):
 
     Returns:
         numpy.ndarray: The discretized values in real space.
-    """    
+    """
     if zeta is None:
         return _trig2real_1d(theta, xm, fmnc, fmns)
     else:
         return _trig2real_2d(theta, zeta, xm, xn, fmnc, fmns)
 
+
 def _trig2real_1d(theta, xm, fmnc=None, fmns=None):
     _mt = np.reshape(xm, (-1, 1)) * theta
     _cos = np.cos(_mt)
@@ -266,7 +334,8 @@ def _trig2real_1d(theta, xm, fmnc=None, fmns=None):
         f += np.matmul(np.reshape(fmnc, (1, -1)), _cos)
     if fmns is not None:
         f += np.matmul(np.reshape(fmns, (1, -1)), _sin)
-    return f.ravel()   
+    return f.ravel()
+
 
 def _trig2real_2d(theta, zeta, xm, xn, fmnc=None, fmns=None):
     npol, ntor = len(theta), len(zeta)
@@ -286,6 +355,41 @@ def _trig2real_2d(theta, zeta, xm, xn, fmnc=None, fmns=None):
     return f.reshape(npol, ntor)
 
 
+def real2trig_2d(f, xm, xn, theta, zeta):
+    """Fourier decomposition in 2D
+
+    Args:
+        f (numpy.ndarray): The 2D function to be decomposed. Size: [npol, ntor].
+        xm (numpy.ndarray): Poloildal mode number. Size: [mn,]
+        xn (numpy.ndarray): Toroildal mode number. Size: [mn,]
+        theta (numpy.ndarray): Poloidal angles. Size:[npol,].
+        zeta (numpy.ndarray): Toroidal angles. Size:[ntor,]
+
+    Returns:
+        numpy.ndarray, numpy.ndarray: Cos harmonics, sin harmonics. Size: [mn,]
+    """
+    npol, ntor = len(theta), len(zeta)
+    assert (npol, ntor) == np.shape(
+        f
+    ), "F function dimension should be consistent with theta, zeta."
+    _tv, _zv = np.meshgrid(theta, zeta, indexing="ij")
+    # mt - nz (in matrix)
+    _mtnz = np.matmul(np.reshape(xm, (-1, 1)), np.reshape(_tv, (1, -1))) - np.matmul(
+        np.reshape(xn, (-1, 1)), np.reshape(_zv, (1, -1))
+    )
+    _cos = np.cos(_mtnz)
+    _sin = np.sin(_mtnz)
+
+    fmnc = np.ravel(np.matmul(_cos, f.reshape(-1, 1)))
+    fmns = np.ravel(np.matmul(_sin, f.reshape(-1, 1)))
+    fac = 2.0 / (npol * ntor)
+    # m=0, n=0 term or m=0 terms?
+    ind = np.logical_and(xm == 0, xn == 0)
+    fmnc[ind] *= 0.5
+    fmns[ind] *= 0.5
+    return fmnc * fac, fmns * fac
+
+
 def vmec2focus(
     vmec_file,
     focus_file="plasma.boundary",
@@ -418,3 +522,133 @@ def vmec2focus(
             )
             # FOCUS uses mu - nv
     return
+
+
+def booz2focus(booz_file, ns=-1, focus_file="plasma.boundary", tol=1e-6, Nfp=1):
+    """convert BOOZ_XFORM output into FOCUS format plasma surface (in Boozer coordinates)
+
+    Args:
+        booz_file (str): Netcdf file of BOOZ_XFORM output
+        ns (int, optional): The specific flux surface you want to convert. Defaults to -1.
+        focus_file (str, optional): FOCUS plasma boundary filename. Defaults to 'plasma.boundary'.
+        tol ([type], optional): Tolerance to truncate. Defaults to 1E-6.
+        Nfp (int, optional): [description]. Defaults to 1.
+    """
+    import xarray
+
+    booz = xarray.open_dataset(booz_file)
+    mn = int(booz["mnboz_b"].values)
+    xm = np.array(booz["ixm_b"])
+    xn = np.array(booz["ixn_b"]) / Nfp
+    rbc = np.array(booz["rmnc_b"][ns, :])
+    # rbs = np.zeros(mn)
+    zbs = np.array(booz["zmns_b"][ns, :])
+    # zbc = np.zeros(mn)
+    pmns = np.array(booz["pmns_b"][ns, :])
+    # pmnc = np.zeros(mn)
+
+    # Nfp = 1
+    Nbnf = 0
+
+    amn = 0
+    for imn in range(mn):
+        if (abs(rbc[imn]) + abs(zbs[imn] + abs(pmns[imn]))) > tol:
+            amn += 1  # number of nonzero coef.
+    with open(focus_file, "w") as fofile:
+        fofile.write("# bmn   bNfp   nbf " + "\n")
+        fofile.write("{:d} \t {:d} \t {:d} \n".format(amn, Nfp, Nbnf))
+        fofile.write("#plasma boundary" + "\n")
+        fofile.write("# n m Rbc Rbs Zbc Zbs Pmnc Pmns" + "\n")
+        for imn in range(mn):
+            if (abs(rbc[imn]) + abs(zbs[imn] + abs(pmns[imn]))) > tol:
+                fofile.write(
+                    "{:4d}  {:4d} \t {:23.15E}  {:12.5E}  {:12.5E}  {:23.15E}  {:12.5E}  {:23.15E} \n".format(
+                        xn[imn], xm[imn], rbc[imn], 0.0, 0.0, zbs[imn], 0.0, pmns[imn]
+                    )
+                )
+        fofile.write("#Bn harmonics \n")
+        fofile.write("# n m bnc bns" + "\n")
+    print("Finished write FOCUS input file at ", focus_file)
+    return
+
+
+def read_focus_boundary(filename):
+    """Read FOCUS/FAMUS plasma boundary file
+
+    Args:
+        filename (str): File name and path.
+
+    Returns:
+        boundary (dict): Dict contains the parsed data.
+            nfp : number of toroidal periods
+            nfou : number of Fourier harmonics for describing the boundary
+            nbn : number of Fourier harmonics for Bn
+            surface : Toroidal surface dict, containing 'xm', 'xn', 'rbc', 'rbs', 'zbc', 'zbs'
+            bnormal : Input Bn dict, containing 'xm', 'xn', 'bnc', 'bns'
+    """
+    boundary = {}
+    surf = {}
+    bn = {}
+    with open(filename, "r") as f:
+        line = f.readline()  # skip one line
+        line = f.readline()
+        num = int(line.split()[0])  # harmonics number
+        nfp = int(line.split()[1])  # number of field periodicity
+        nbn = int(line.split()[2])  # number of Bn harmonics
+        boundary["nfp"] = nfp
+        boundary["nfou"] = num
+        boundary["nbn"] = nbn
+        # read boundary harmonics
+        xm = []
+        xn = []
+        rbc = []
+        rbs = []
+        zbc = []
+        zbs = []
+        line = f.readline()  # skip one line
+        line = f.readline()  # skip one line
+        for i in range(num):
+            line = f.readline()
+            line_list = line.split()
+            n = int(line_list[0])
+            m = int(line_list[1])
+            xm.append(m)
+            xn.append(n)
+            rbc.append(float(line_list[2]))
+            rbs.append(float(line_list[3]))
+            zbc.append(float(line_list[4]))
+            zbs.append(float(line_list[5]))
+        surf["xm"] = np.array(xm)
+        surf["xn"] = np.array(xn)
+        surf["rbc"] = np.array(rbc)
+        surf["rbs"] = np.array(rbs)
+        surf["zbc"] = np.array(zbc)
+        surf["zbs"] = np.array(zbs)
+        boundary["surface"] = surf
+        # read Bn fourier harmonics
+        xm = []
+        xn = []
+        bnc = []
+        bns = []
+        if nbn > 0:
+            line = f.readline()  # skip one line
+            line = f.readline()  # skip one line
+        for i in range(nbn):
+            line = f.readline()
+            line_list = line.split()
+            n = int(line_list[0])
+            m = int(line_list[1])
+            xm.append(m)
+            xn.append(n)
+            bnc.append(float(line_list[2]))
+            bns.append(float(line_list[3]))
+        bn["xm"] = np.array(xm)
+        bn["xn"] = np.array(xn)
+        bn["bnc"] = np.array(bnc)
+        bn["bns"] = np.array(bns)
+        boundary["bnormal"] = bn
+    return boundary
+
+
+def div0(a, b):
+    return np.divide(a, b, out=np.zeros_like(a), where=b != 0)
